/*
   Unix SMB/Netbios implementation.
   Version 1.9.
   start/stop nmbd and smbd
   Copyright (C) Andrew Tridgell 1998

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 * ??-???-????	Jean-Yves Collot?	SAMBA 2.2.8
 * 25-Sep-2004	J. Malmberg		Update for VMS 8.2
 */

#include "includes.h"
#include "smb.h"

#ifdef __VMS
#include <descrip.h>

int lib$spawn
  (const struct dsc$descriptor_s * const cmd,
   const struct dsc$descriptor_s * const input_file,
   const struct dsc$descriptor_s * const output_file,
   const unsigned long * flags,
   const struct dsc$descriptor_s * const process_name,
   int * process_id,
   int * comp_status,
   const unsigned char * efn,
   int * (astadr)(int * astarg),
   int * * astarg,
   const struct dsc$descriptor_s * const prompt_str,
   const struct dsc$descriptor_s * const cli,
   const struct dsc$descriptor_s * const table);

#endif


/* need to wait for daemons to startup */
#define SLEEP_TIME 3

/* startup smbd */
void start_smbd(void)
{
	pstring binfile;

	if (geteuid() != 0) return;

#ifdef __VMS
	{
	    struct dsc$descriptor_s dsc;
	    const char *prg = "$ Run/detached/process=smbd samba_root:[bin]smbd";
	    dsc.dsc$w_length = strlen(prg);
	    dsc.dsc$a_pointer = (char *) prg;
	    dsc.dsc$b_class = DSC$K_CLASS_S;
	    dsc.dsc$b_dtype = DSC$K_DTYPE_T;
	    lib$spawn (
              	  &dsc, /* command */
              	  0,	/* input_file */
              	  0, /* output_file */
              	  0,	/* flags */
              	  0,	/* proc_name */
              	  0,	/* proc_id */
              	  0, /* comp_status */
		  0,	/* efn */
		  0,	/* ast() */
		  0,	/* ast arg */
		  0,	/* prompt_str */
		  0,	/* cli_str */
		  0);   /* table */
	    sleep(SLEEP_TIME);
	}
#else
	if (fork()) {
		sleep(SLEEP_TIME);
		return;
	}

	slprintf(binfile, sizeof(pstring) - 1, "%s/smbd", SBINDIR);

	become_daemon();

	execl(binfile, binfile, "-D", NULL);

	exit(0);
#endif
}

/* startup nmbd */
void start_nmbd(void)
{
	pstring binfile;

	if (geteuid() != 0) return;

#ifdef __VMS
	{
	    struct dsc$descriptor_s dsc;
	    const char *prg = "$ Run/detached/process=nmbd samba_root:[bin]nmbd";
	    dsc.dsc$w_length = strlen(prg);
	    dsc.dsc$a_pointer = (char *) prg;
	    dsc.dsc$b_class = DSC$K_CLASS_S;
	    dsc.dsc$b_dtype = DSC$K_DTYPE_T;
	    lib$spawn (
              	  &dsc, /* command */
              	  0,	/* input_file */
              	  0, /* output_file */
              	  0,	/* flags */
              	  0,	/* proc_name */
              	  0,	/* proc_id */
              	  0, /* comp_status */
		  0,	/* efn */
		  0,	/* ast() */
		  0,	/* ast arg */
		  0,	/* prompt_str */
		  0,	/* cli_str */
		  0);   /* table */
	    sleep(SLEEP_TIME);
	}
#else
	if (fork()) {
		sleep(SLEEP_TIME);
		return;
	}

	slprintf(binfile, sizeof(pstring) - 1, "%s/nmbd", SBINDIR);

	become_daemon();

	execl(binfile, binfile, "-D", NULL);

	exit(0);
#endif
}


/* stop smbd */
void stop_smbd(void)
{
	pid_t pid = pidfile_pid("smbd");

	if (geteuid() != 0) return;

	if (pid == 0) return;

	kill(pid, SIGTERM);
}

/* stop nmbd */
void stop_nmbd(void)
{
	pid_t pid = pidfile_pid("nmbd");

	if (geteuid() != 0) return;

	if (pid == 0) return;

	kill(pid, SIGTERM);
}

/* kill a specified process */
void kill_pid(pid_t pid)
{
	if (geteuid() != 0) return;

	if (pid <= 0) return;

	kill(pid, SIGTERM);
	sleep(SLEEP_TIME);
}
